package com.ruoyi.bst.gateway.service.impl;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.TimeUnit;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.ruoyi.bst.gateway.domain.Gateway;
import com.ruoyi.bst.gateway.domain.GatewayVO;
import com.ruoyi.bst.gateway.mapper.GatewayMapper;
import com.ruoyi.bst.gateway.service.GatewayIotService;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.collection.CollectionUtils;
import com.ruoyi.iot.constants.IotConstants;
import com.ruoyi.iot.domain.IotDeviceInfo;
import com.ruoyi.iot.domain.response.CommandResponse;
import com.ruoyi.iot.service.IotService;
import com.ruoyi.iot.service.impl.DeviceOnlineStatus;

import lombok.extern.slf4j.Slf4j;

@Service
@Slf4j
public class GatewayIotServiceImpl implements GatewayIotService {

    @Autowired
    private IotService iotService;

    @Autowired
    private ScheduledExecutorService scheduledExecutorService;

    @Autowired
    private GatewayMapper gatewayMapper;
    
    @Override
    public void refresh(List<GatewayVO> gatewayList, String onlineType) {
        if (CollectionUtils.isEmptyElement(gatewayList)) {
            return;
        }

        // 获取物联网信息
        List<IotDeviceInfo> iotList = iotService.getDeviceInfo(gatewayList);
        if (CollectionUtils.isEmptyElement(iotList)) {
            return;
        }

        // 更新设备信息
        for (GatewayVO gateway : gatewayList) {
            if (gateway == null || StringUtils.isBlank(gateway.getMac())) {
                continue;
            }
            IotDeviceInfo iot = iotList.stream()
                    .filter(item -> Objects.equals(item.getMac(), gateway.getMac()))
                    .findFirst().orElse(null);
            this.setIotInfo(gateway, iot);
        }

        // 异步更新数据库
        scheduledExecutorService.schedule(() -> {
            for (GatewayVO gateway : gatewayList) {
                try {
                    // 获取在线状态
                    if (StringUtils.hasText(onlineType)) {
                        String onlineStatus = iotService.getOnlineStatus(gateway, onlineType);
                        this.setOnlineStatus(gateway, onlineStatus);
                    }
                } catch (Exception e) {
                    log.error("获取{}物联网在线状态失败", gateway.getMac(), e);
                }
            }
            // 更新物联网信息
            this.updateIot(gatewayList);
        }, 0, TimeUnit.SECONDS);
    }

    private void setIotInfo(GatewayVO gateway, IotDeviceInfo iot) {
        if (iot == null) {
            return;
        }
        gateway.setImei(iot.getImei());
    }
    
    private void setOnlineStatus(GatewayVO gateway, String onlineStatus) {
        // 是否在线
        gateway.setOnlineStatus(onlineStatus);
        if (DeviceOnlineStatus.ONLINE.getStatus().equals(onlineStatus)) {
            gateway.setLastOnlineTime(LocalDateTime.now());
        }
    }

    public int updateIot(List<GatewayVO> gatewayList) {
        if (gatewayList == null || gatewayList.isEmpty()) {
            return 0;
        }
        List<Gateway> updateList = new ArrayList<>();
        for (GatewayVO gateway : gatewayList) {
            if (gateway == null || gateway.getId() == null) {
                continue;
            }
            Gateway data = new Gateway();
            data.setId(gateway.getId());
            data.setImei(gateway.getImei());
            data.setOnlineStatus(gateway.getOnlineStatus());
            data.setLastOnlineTime(gateway.getLastOnlineTime());
            updateList.add(data);
        }
        return gatewayMapper.batchUpdate(updateList);
    }

    @Override
    public CommandResponse send(String gatewayMac, String deviceMac, String command, Integer timeout) {
        if (StringUtils.isBlank(gatewayMac) || StringUtils.isBlank(deviceMac) || StringUtils.isBlank(command)) {
            return null;
        }
        StringBuilder sb = new StringBuilder();
        sb.append(IotConstants.COMMAND_START)
            .append(deviceMac)
            .append(IotConstants.COMMAND_SEPARATOR)
            .append(command)
            .append(IotConstants.COMMAND_END);
        Gateway gateway = new Gateway();
        gateway.setMac(gatewayMac);
        return iotService.sendCommand(gateway, sb.toString(), timeout);
    }
}
